/**
 * @author Calcurates Team
 * @copyright Copyright © 2019 Calcurates (https://www.calcurates.com)
 * @license https://opensource.org/licenses/osl-3.0.php Open Software License (OSL 3.0)
 * @package Calcurates_ModuleMagento
 */

define([
    'ko',
    'jquery',
    'underscore',
    'uiElement',
    'Calcurates_ModuleMagento/js/model/product/rates',
    'mage/translate',
    'loader'
], function (ko, $, _, Element, ratesModel, $t) {
    'use strict';

    return Element.extend({
        defaults: {
            rates: [],
            storeCode: '',
            productId: 0,
            fallbackMessage: '',
            isLoggedIn: 0,
            googlePlacesEnabled: false,
            googlePlacesApiKey: null,
            googlePlacesAutocompleteInstance: null,
            placesAutocompleteUrl: 'https://maps.googleapis.com/maps/api/js?key=API_KEY&libraries=places&callback=initAutocomplete',
            selectors: {
                addToCartForm: 'form#product_addtocart_form',
                ratesContainer: '[data-calcurates-js="rates-container"]'
            },
            nodes: {
                ratesContainer: null,
                addToCartForm: null
            },
            rateTemplatesMap: {
                free_shipping: 'Calcurates_ModuleMagento/product/rate/default',
                flat_rate: 'Calcurates_ModuleMagento/product/rate/default',
                table_rates: 'Calcurates_ModuleMagento/product/rate/default',
                in_store_pickup: 'Calcurates_ModuleMagento/product/rate/default',
            }
        },

        initialize: function () {
            this._super();
            window.initAutocomplete = this.initPlacesAutocompleteFields.bind(this)
            this._initNodes();
            return this;
        },

        initObservable: function () {
            this._super();

            this.rates = ratesModel.rates;

            return this;
        },

        getRateTemplate: function (rate) {
            return this.rateTemplatesMap[rate.type];
        },

        _initNodes: function () {
            this.nodes.ratesContainer = $(this.selectors.ratesContainer);
            this.nodes.addToCartForm = $(this.selectors.ratesContainer);

            if (!this.usePlacesAutocomplete()) {
                ratesModel.loadLocations(this.storeCode, [this.productId], this.isLoggedIn);
            }
        },

        usePlacesAutocomplete: function () {
            return this.googlePlacesEnabled && this.googlePlacesApiKey;
        },

        initPlacesAutocomplete: function () {
            if (this.usePlacesAutocomplete()) {
                require([this.placesAutocompleteUrl.replace('API_KEY', this.googlePlacesApiKey)]);
            }
        },

        initPlacesAutocompleteFields: function () {
            const input = document.getElementById("placesAutocomplete");
            const options = {
                fields: ["address_components", "formatted_address", "geometry", "name"],
                types: ["address"],
            };
            this.googlePlacesAutocompleteInstance = new google.maps.places.Autocomplete(input, options);
            this.googlePlacesAutocompleteInstance.addListener('place_changed', this.onPlaceChanged.bind(this))
        },

        onPlaceChanged: function () {
            const place = this.googlePlacesAutocompleteInstance.getPlace();
            if (!place.geometry) {
                document.getElementById("placesAutocomplete").value = '';
            } else {
                document.getElementById("placesAutocomplete").value = place.formatted_address;
                ratesModel.loadLocations(
                    this.storeCode,
                    [this.productId],
                    this.isLoggedIn,
                    this.convertPlaceToAddress(place)
                );

            }
        },

        convertPlaceToAddress: function (place) {
            let address = {},
                mapper = {
                    country: 'country',
                    regionCode: 'administrative_area_level_1',
                    postalCode: 'postal_code',
                    city: 'locality',
                    addressLine1: 'route',
                    streetNumber: 'street_number',
                }
            if (place.address_components && _.isArray(place.address_components)) {
                place.address_components.forEach(function (component) {
                    _.each(mapper, function (value, key) {
                        if (_.contains(component.types, value)) {
                            address[key] = key === 'country' ? component.short_name : component.long_name
                        }
                    })
                })
            }
            if (address['addressLine1'] && address['streetNumber']) {
                address.addressLine1 = address['streetNumber'] + ' ' + address['addressLine1']
            }
            delete address['streetNumber']
            return address
        }

    });
});
